var request = require('request');

function PaysafecardPayoutController(key = '', environment = 'PRODUCTION') {
  // Construct payment class.
  this.key = key; // define psc key
  this.error = {};
  this.environment = environment;
  setEnvironment.call(this);
}

// Prepare and start request
function doRequest(parameter, method, headers, success) {
  /*
  Make an API request and return all needed information
  */

  // set general request options
  var options = {};
  options.port = 443;
  options.timeout = 60 * 1000; // timeout in ms: 60.000ms = 60s
  options.strictSSL = false;
  options.json = true;
  options.uri = this.url;

  // define default headers
  if (!headers) {
    headers = {};
  }
  headers['Authorization'] = 'Basic ' + new Buffer(this.key).toString('base64');
  options.headers = headers;

  this.requestParameter = parameter;

  // method handling
  if (method == 'POST') {
    options.method = 'POST';
    options.body = parameter;
  } else if (method == 'GET') {
    options.method = 'GET';
    if (parameter) {
      options.uri = this.url + parameter;
    }
  }

  // start request with options
  request(options, function (error, response, body) {
    // when request is complete
    // asign requested data

    this.request = response;
    this.requestinfo = {
      'request_error': error,
      'request_headers': response.request.headers,
      'response': response,
      'response_headers': response.headers,
      'status_code': response.statusCode,
      'status_message': response.statusMessage
    };

    // reset url
    setEnvironment.call(this);

    // push response to success
    if(success){
      success();
    }

  }.bind(this));
}

PaysafecardPayoutController.prototype.requestIsOk = function() {
  // check if request was successful
  if (this.request.statusCode < 300) {
    return true;
  } else {
    return false;
  }
};

PaysafecardPayoutController.prototype.getResponse = function() {
  // get the response
  if (!this.request.body){
    return {};
  } else {
    return this.request.body;
  }
};

PaysafecardPayoutController.prototype.getRequestPrameter = function() {
  // get request parameter
  return this.requestParameter;
};

PaysafecardPayoutController.prototype.getRequestInfo = function() {
  // get request information
  return this.requestinfo;
};

function setEnvironment() {
  // set the environment and URLs
  if (this.environment == 'TEST') {
    this.url = 'https://apitest.paysafecard.com/v1/payouts/';
  } else if (this.environment == 'PRODUCTION') {
    this.url = 'https://api.paysafecard.com/v1/payouts/';
  } else {
    console.log('#### environment not supported');
  }
}

PaysafecardPayoutController.prototype.getPayoutDetail = function(payment_id, success) {
  // get payout Details
  this.url = this.url + payment_id;

  doRequest.call(this, null, 'GET', null, function() {
    // return false if request failed
    if (this.requestIsOk) {
      success(this.getResponse());
    } else {
      success(false);
    }
  });
};

PaysafecardPayoutController.prototype.getLimits = function(currency, success) {
  // get Limits
  this.url = this.url + 'limits/' + currency;

  doRequest.call(this, null, 'GET', null, function() {
    // return false if request failed
    if (this.requestIsOk) {
      success(this.getResponse());
    } else {
      success(false);
    }
  });
};

PaysafecardPayoutController.prototype.validatePayout = function(options, success) {
  // validate a payout
  /*
  necessary variables:
  options.amount = 10.00 # payout amount
  options.currency = 'EUR' # payout currency, USD, EUR, SEK..
  options.customer_id = 'customer_id_123' # your psc customer id, merchant client id
  options.customer_email = 'ABCEXAMPLEABC@ABCEXAMPLEABC.ABC'
  options.customer_ip = '123.123.123.123' # the customers IP
  options.first_name = 'John' #  first name
  options.last_name = 'Doe' #  last name
  options.birthday = '1986-06-16' # day of birth

  optional variables:
  options.correlation_id = uuid.v4() # Define a unique identifier for referencing (optional) default is None
  options.submerchant_id = 1 # Reporting criteria
  */

  var headers = {};

  var customer = {
    'id': options.customer_id,
    'email': options.customer_email,
    'first_name': options.first_name,
    'last_name': options.last_name,
    'date_of_birth': options.birthday,
    'ip': options.customer_ip
  };

  var parameter = {
    'amount': options.amount,
    'currency': options.currency,
    'type': 'PAYSAFECARD',
    'customer': customer,
    'capture': false
  };

  if (options.submerchant_id) {
    parameter.submerchant_id = options.submerchant_id;
  }

  if (options.correlation_id) {
    headers['Correlation-ID'] = options.correlation_id;
  }

  doRequest.call(this, parameter, 'POST', headers, function() {
    // return false if request failed
    if (this.requestIsOk) {
      success(this.getResponse());
    } else {
      success(false);
    }
  });
};

PaysafecardPayoutController.prototype.executePayout = function(options, success) {
  // execute a payout
  /*
  necessary variables:
  options.amount = 10.00 # payout amount
  options.currency = 'EUR' # payout currency, USD, EUR, SEK..
  options.customer_id = 'customer_id_123' # your psc customer id, merchant client id
  options.customer_email = 'ABCEXAMPLEABC@ABCEXAMPLEABC.ABC'
  options.customer_ip = '123.123.123.123' # the customers IP
  options.first_name = 'John' #  first name
  options.last_name = 'Doe' #  last name
  options.birthday = '1986-06-16' # day of birth

  optional variables:
  options.correlation_id = uuid.v4() # Define a unique identifier for referencing (optional) default is None
  options.submerchant_id = 1 # Reporting criteria
  */

  var headers = {};

  var customer = {
    'id': options.customer_id,
    'email': options.customer_email,
    'first_name': options.first_name,
    'last_name': options.last_name,
    'date_of_birth': options.birthday,
    'ip': options.customer_ip
  };

  var parameter = {
    'amount': options.amount,
    'currency': options.currency,
    'type': 'PAYSAFECARD',
    'customer': customer,
    'capture': true
  };

  if (options.submerchant_id) {
    parameter.submerchant_id = options.submerchant_id;
  }

  if (options.correlation_id) {
    headers['Correlation-ID'] = options.correlation_id;
  }

  doRequest.call(this, parameter, 'POST', headers, function() {
    // return false if request failed
    if (this.requestIsOk) {
      success(this.getResponse());
    } else {
      success(false);
    }
  });
};

PaysafecardPayoutController.prototype.getError = function() {
  // get Errors request errors and returned errors

  if(!this.getResponse().number) {
    switch(this.request.statusCode) {
      case 400:
        this.error.number = 'HTTP:400';
        this.error.message = 'Logical error. The requested URL cannot be found. Check your request data';
        break;
      case 403:
        this.error.number = 'HTTP:403';
        this.error.message = 'Transaction could not be initiated due to connection problems. The servers IP address is probably not whitelisted!';
        break;
      case 500:
        this.error.number = 'HTTP:500';
        this.error.message = 'Server error.';
        break;
    }

    if (this.error) {
      return this.error;
    }
  }

  switch(this.getResponse()) {
    case 3162:
      this.error.number = this.getResponse().number;
      this.error.message = 'Unfortunately, no my paysafecard account exists under the e-mail address you have entered. Please check the address for a typing error. If you do not have a my paysafecard account, you can register for one online now for free.';
      break;
    case 3195:
      this.error.number = this.getResponse().number;
      this.error.message = 'The personal details associated with your my paysafecard account do not match the details of this account. Please check the first names, surnames and dates of birth entered in both accounts and request the payout again.';
      break;
    case 3234:
      this.error.number = this.getResponse().number;
      this.error.message = 'Unfortunately, the payout could not be completed due to a problem which has arisen with your my paysafecard account. paysafecard has already sent you an e-mail with further information on this. Please follow the instructions found in this e-mail before requesting the payout again.';
      break;
    case 3198:
      this.error.number = this.getResponse().number;
      this.error.message = 'Unfortunately, the payout could not be completed due to a problem which has arisen with your my paysafecard account. Please contact the paysafecard support team. info@paysafecard.com';
      break;
    case 10008:
      this.error.number = this.getResponse().number;
      this.error.message = 'Invalid API Key';
      break;
    default:
      this.error.number = this.getResponse().number;
      this.error.message = this.getResponse().message;
      // this.error.message = 'Unfortunately there has been a technical problem and your payout request could not be executed. If the problem persists, please contact our customer support: support@company.com';
      break;
  }

  return this.error;
};

module.exports = PaysafecardPayoutController;
